<?php

namespace Drupal\element451_forms\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Configure Element API settings for this site.
 */
class ElementApiSettingsForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'element_api_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['element451_forms.api_settings'];
  }

  /**
   * Build the configuration form.
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('element451_forms.api_settings');

    // Load keys from the Keys module.
    $key_options = \Drupal::service('key.repository');
    $key_list = $key_options->getKeyNamesAsOptions();

    //Base URL
    $form['base_url'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Base URL'),
      '#description' => $this->t('Add the base URL for your Element451 API. This value should look something like this: <strong>https://xxxx.api.451.io</strong> with no trailing slash.'),
      '#default_value' => $config->get('base_url'),
      '#required' => TRUE,
    ];

    // Bearer Token key.
    $form['bearer_token'] = [
      '#type' => 'select',
      '#title' => $this->t('Bearer Token'),
      '#description' => $this->t('Select a key for the Bearer Token from the Keys module.'),
      '#options' => $key_list,
      '#default_value' => $config->get('bearer_token'),
      '#required' => TRUE,
    ];

    // Analytics Key.
    $form['analytics_key'] = [
      '#type' => 'select',
      '#title' => $this->t('Analytics Key'),
      '#description' => $this->t('Select a key for the Analytics Key from the Keys module.'),
      '#options' => $key_list,
      '#default_value' => $config->get('analytics_key'),
      '#required' => TRUE,
    ];

    // Feature Key.
    $form['feature_key'] = [
      '#type' => 'select',
      '#title' => $this->t('Feature Key'),
      '#description' => $this->t('Select a key for the Feature Key from the Keys module.'),
      '#options' => $key_list,
      '#default_value' => $config->get('feature_key'),
      '#required' => TRUE,
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    // Save the selected keys in the configuration.
    $this->config('element451_forms.api_settings')
      ->set('base_url', $form_state->getValue('base_url'))
      ->set('bearer_token', $form_state->getValue('bearer_token'))
      ->set('analytics_key', $form_state->getValue('analytics_key'))
      ->set('feature_key', $form_state->getValue('feature_key'))
      ->save();

    parent::submitForm($form, $form_state);
  }
}
