<?php
namespace Drupal\element451_forms\Plugin\WebformHandler;

use Drupal\webform\Plugin\WebformHandlerBase;
use Drupal\webform\WebformSubmissionInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Annotation\Translation;
use Drupal\Component\Annotation\Plugin;

/**
 * Form submission handler.
 *
 * @WebformHandler(
 *   id = "element_connector",
 *   label = @Translation("Element Connector"),
 *   category = @Translation("Form Handler"),
 *   description = @Translation("Creates an API connection with Element for Form Submissions"),
 *   cardinality = \Drupal\webform\Plugin\WebformHandlerInterface::CARDINALITY_SINGLE,
 *   results = \Drupal\webform\Plugin\WebformHandlerInterface::RESULTS_PROCESSED,
 * )
 */

class ElementConnector extends WebformHandlerBase {


  /**
   * {@inheritdoc}
   */
  public function submitWebformSubmission(WebformSubmissionInterface $webform_submission, $update = TRUE) {
    // Add custom submission handling logic here.
    \Drupal::logger('custom_webform_handler')->notice('Submission processed.');
  }

  /**
   * {@inheritdoc}
   */
  public function preSave(WebformSubmissionInterface $webform_submission, $update = TRUE) {
    $values = $webform_submission->getData();
    $fields_for_element = [];
    $webform = $this->webform;
    $fields = $webform->getElementsDecoded();
    $sub_fields = [];

    //Grab data for Element Post Request
    foreach($values as $value_key => $value_value){
      if(!empty($fields[$value_key]['#custom_values']['slug']) && !empty($fields[$value_key]['#custom_values']['name'])) {
        if($value_value !== '' && $value_value !== NULL){
          $field_object = (object)[
            'slug' => $fields[$value_key]['#custom_values']['slug'],
            'name' => $fields[$value_key]['#custom_values']['name'],
            'value' => $value_value,
          ];
          if ($fields[$value_key]['#type'] === 'checkboxes') {
            $value_string = '';
            foreach ($value_value as $multiple_value) {
              $value_string = $value_string . $multiple_value . ',';
            }
            $value_string = rtrim($value_string, ',');
            $field_object->value = $value_string;
          }
          array_push($fields_for_element, $field_object);
        }
      }

     //Handle subfields
      if(!empty($fields[$value_key]['#custom_values']) && $fields[$value_key]['#custom_values']['slug'] === NULL && !empty($fields[$value_key]['#custom_values']['parent_name'])) {
       if($value_value !== "" && $value_value !== NULL){
         $sub_field_values = (object)[
           'name' => $fields[$value_key]['#custom_values']['name'],
           'value' => $value_value,
         ];
         $key = $fields[$value_key]['#custom_values']['parent_name'];
         if (!empty($sub_fields[$key])) {
           $sub = $sub_fields[$key]->subfields;
           //Convert to array to merge, then back to object for saving
           $sub_array = (array)$sub;
           $sub_field_values_array = $sub_field_values;
           array_push($sub_array, $sub_field_values_array);
           $sub_fields[$key]->subfields = $sub_array;
         } else {
           $sub_fields[$key] = (object)[
             'slug' => $fields[$value_key]['#custom_values']['parent_slug'],
             'name' => $fields[$value_key]['#custom_values']['parent_name'],
             'subfields' => (object)[$sub_field_values],
           ];
         }
        }
      }
    }

    //Set subfields to array
    foreach($sub_fields as $sub_field_key => $sub_field_value){
      array_push($fields_for_element, $sub_field_value);
    }


    $url = $_SERVER['HTTP_REFERER'];

    $item = (object) [
      'fields_id' => $webform->getThirdPartySetting('element', 'element451_fields_id'),
      'initial_form' => TRUE,
      'source_url' => '%27' . urlencode($url) . '%27',
      'fields' => $fields_for_element
    ];

    $encode = json_encode($item);
    $element451_api = \Drupal::service('element451_forms.element_api');
    $response = $element451_api->postElementSubmission($webform->getThirdPartySetting('element', 'element451_form_id'), $encode);

    if($response){
      $values['status'] = 'Successful';
    }
    else{
      $values['status'] = 'Failed';
      $this->sendFailureEmail($webform_submission);
    }

    $webform_submission->setData($values);

  }

  /**
   * {@inheritdoc}
   */
  public function getSummary() {

    $settings = $this->getSettings();
    $summary = '';

    foreach($settings as $key => $value){
       if(!empty($value)){
         $summary = $summary . '<strong>' . $key . '</strong>: ' . $value . '<br>';
       }
    }

    return [
        '#markup' => $summary,
      ];
  }

  /**
   * Send an email notification to the site admin on failure.
   */
  protected function sendFailureEmail(WebformSubmissionInterface $webform_submission) {
    // Get the site email.
    $site_mail = \Drupal::config('system.site')->get('mail');

    // Email subject and body.
    $mail_subject = t('Webform Submission Failed');
    $mail_body = t('A submission failed on the @webform webform. This form is connected to Element451.
    Please look into the failed submission. @url', [
      '@webform' => $webform_submission->getWebform()->label(),
      '@url' => $webform_submission->getWebform()->toUrl('canonical', ['absolute' => TRUE])->toString(),
    ]);

    // Prepare the mail parameters.
    $module = 'element451_forms';  // The module sending the email.
    $key = 'webform_submission_failed';  // A unique identifier for this email type.
    $to = $site_mail;  // Send to site admin.
    $params = [
      'subject' => $mail_subject,
      'body' => $mail_body,
      'reply-to' => $site_mail,
    ];
    $langcode = \Drupal::currentUser()->getPreferredLangcode();
    $send = TRUE;

    // Send the email using Drupal's mail manager service.
    $mail_manager = \Drupal::service('plugin.manager.mail');
    $result = $mail_manager->mail($module, $key, $to, $langcode, $params, NULL, $send);

    if ($result['result'] !== TRUE) {
      \Drupal::logger('custom_module')->error('There was a problem sending the failure notification email.');
    }
  }

}
